import logging
from typing import Optional

import requests

from GoogleTTS.GoogleTTS.utils import *

# Logger
log = logging.getLogger(__name__)
log.addHandler(logging.NullHandler())


class GoogleTTS:
    def __init__(self):
        self.url = "https://content-texttospeech.googleapis.com/v1/text:synthesize"
        self.x_referer = "https://explorer.apis.google.com"
        self.key: str = "AIzaSyAa8yy0GdcGPHdtD083HiGGx_S0vMPScDM"
        self.language_code: str = "vi-VN"
        self.ssml_gender: SsmlGender = SsmlGender.FEMALE
        self.audio_encoding: AudioEncoding = AudioEncoding.MP3
        self.sample_rate_hertz: int = 48000
        self.speaking_rate: float = 1.0

    def set_key(self, key: str):
        self.key = key

    def set_language_code(self, lang_code: str):
        self.language_code = lang_code

    def set_ssml_gender(self, gender: SsmlGender):
        self.ssml_gender = gender

    def set_audio_encoding(self, encoding: AudioEncoding):
        self.audio_encoding = encoding

    def set_sample_rate(self, rate: int):
        self.sample_rate_hertz = rate

    def set_speaking_rate(self, rate: float):
        self.speaking_rate = rate

    def __prepare_body(self, text: str) -> dict:
        _input: dict = {"text": text}
        _voice: dict = {
            "languageCode": self.language_code,
            "ssmlGender": self.ssml_gender
        }
        _audioConfig: dict = {
            "audioEncoding": self.audio_encoding,
            "sampleRateHertz": self.sample_rate_hertz,
            "speakingRate": self.speaking_rate
        }
        body: dict = {
            "input": _input,
            "voice": _voice,
            "audioEncoding": _audioConfig
        }
        return body

    def __prepare_params(self) -> dict:
        params: dict = {
            "alt": "json",
            "key": self.key
        }
        return params

    def __prepare_headers(self) -> dict:
        headers: dict = {
            "x-referer": self.x_referer
        }
        return headers

    def __save_to_file(self, save_file: str, data: bytes):
        index = save_file.rfind(".")
        if index != -1:
            extension = save_file[index:].lower()
            if self.audio_encoding != AudioEncoding.UNSPECIFIED and extension != extensions[self.audio_encoding]:
                log.warning(f"Recommend using the extension '{extensions[self.audio_encoding]}' for encoding '{self.audio_encoding}'")
        else:
            log.warning(f"Cannot find extension of file '{save_file}'!!!")
        with open(save_file, "wb") as f:
            f.write(data)

    def tts(self, text: str, save_file: Optional[str] = None) -> dict:
        headers = self.__prepare_headers()
        body = self.__prepare_body(text=text)
        params = self.__prepare_params()
        r = requests.post(self.url, json=body, params=params, headers=headers)
        if r.status_code == 200:
            res_body = r.json()
            audio_content = res_body["audioContent"]
            audio_bytes = b64_to_audio(b64=audio_content)
            if save_file is not None:
                self.__save_to_file(save_file=save_file, data=audio_bytes)
                return audio_content
        else:
            return r.json()
